@@ -40,6 +40,7 @@ DataManager::DataManager(const Account &account)
 
 DataManager::~DataManager()
 {
+    emit aboutToDestroy();
     Q_FOREACH(SeafileApiRequest *req, reqs_)
     {
         req->deleteLater();
@@ -134,6 +135,21 @@ void DataManager::removeDirent(const QString &repo_id,
     reqs_.push_back(req);
 }
 
+void DataManager::removeDirents(const QString &repo_id,
+                                const QString &parent_path,
+                                const QStringList &filenames)
+{
+    RemoveDirentsRequest *req = new RemoveDirentsRequest(account_, repo_id, parent_path,
+                                                         filenames);
+    connect(req, SIGNAL(success()),
+            SLOT(onRemoveDirentsSuccess()));
+
+    connect(req, SIGNAL(failed(const ApiError&)),
+            SIGNAL(removeDirentsFailed(const ApiError&)));
+    req->send();
+    reqs_.push_back(req);
+}
+
 void DataManager::shareDirent(const QString &repo_id,
                               const QString &path,
                               bool is_file)
@@ -241,6 +257,17 @@ void DataManager::onRemoveDirentSuccess()
     emit removeDirentSuccess(req->path());
 }
 
+void DataManager::onRemoveDirentsSuccess()
+{
+    RemoveDirentsRequest *req = qobject_cast<RemoveDirentsRequest*>(sender());
+
+    if(req == NULL)
+        return;
+
+    removeDirentsCache(req->repoId(), req->parentPath(), false);
+    emit removeDirentsSuccess(req->parentPath(), req->filenames());
+}
+
 void DataManager::onCopyDirentsSuccess()
 {
     emit copyDirentsSuccess();
@@ -287,6 +314,7 @@ FileDownloadTask* DataManager::createDownloadTask(const QString& repo_id,
         account_, repo_id, path, local_path);
     connect(task, SIGNAL(finished(bool)),
             this, SLOT(onFileDownloadFinished(bool)), Qt::UniqueConnection);
+    setupTaskCleanup(task);
 
     return task;
 }
@@ -297,6 +325,7 @@ FileDownloadTask* DataManager::createSaveAsTask(const QString& repo_id,
 {
     FileDownloadTask* task = TransferManager::instance()->addDownloadTask(
         account_, repo_id, path, local_path, true);
+    setupTaskCleanup(task);
 
     return task;
 }
@@ -330,13 +359,19 @@ FileUploadTask* DataManager::createUploadTask(const QString& repo_id,
     else
         task = new FileUploadDirectoryTask(account_, repo_id, parent_dir,
                                            local_path, name);
-
     connect(task, SIGNAL(finished(bool)),
             this, SLOT(onFileUploadFinished(bool)));
+    setupTaskCleanup(task);
 
     return task;
 }
 
+void DataManager::setupTaskCleanup(FileNetworkTask *task)
+{
+    connect(this, SIGNAL(aboutToDestroy()),
+            task, SLOT(cancel()));
+}
+
 FileUploadTask* DataManager::createUploadMultipleTask(const QString& repo_id,
                                                       const QString& parent_dir,
                                                       const QString& local_path,
@@ -348,6 +383,7 @@ FileUploadTask* DataManager::createUploadMultipleTask(const QString& repo_id,
 
     connect(task, SIGNAL(finished(bool)),
             this, SLOT(onFileUploadFinished(bool)));
+    setupTaskCleanup(task);
 
     return task;
 }
