@@ -20,6 +20,7 @@ class GetRepoRequest;
 class CreateSubrepoRequest;
 class DirentsCache;
 class FileCache;
+class FileNetworkTask;
 class FileUploadTask;
 class FileDownloadTask;
 
@@ -58,6 +59,10 @@ public:
                       const QString &path,
                       bool is_file);
 
+    void removeDirents(const QString &repo_id,
+                       const QString &parent_path,
+                       const QStringList &filenames);
+
     void shareDirent(const QString &repo_id,
                      const QString &path,
                      bool is_file);
@@ -113,6 +118,8 @@ public:
     void createSubrepo(const QString &name, const QString& repo_id, const QString &path);
 
 signals:
+    void aboutToDestroy();
+
     void getDirentsSuccess(bool current_readonly, const QList<SeafDirent>& dirents);
     void getDirentsFailed(const ApiError& error);
 
@@ -128,6 +135,9 @@ signals:
     void removeDirentSuccess(const QString& path);
     void removeDirentFailed(const ApiError& error);
 
+    void removeDirentsSuccess(const QString& parent_path, const QStringList& filenames);
+    void removeDirentsFailed(const ApiError& error);
+
     void shareDirentSuccess(const QString& link);
     void shareDirentFailed(const ApiError& error);
 
@@ -149,6 +159,7 @@ private slots:
     void onLockFileSuccess();
     void onRenameDirentSuccess();
     void onRemoveDirentSuccess();
+    void onRemoveDirentsSuccess();
     void onCopyDirentsSuccess();
     void onMoveDirentsSuccess();
 
@@ -159,6 +170,7 @@ private:
     void removeDirentsCache(const QString& repo_id,
                             const QString& path,
                             bool is_file);
+    void setupTaskCleanup(FileNetworkTask *task);
     const Account account_;
 
     QScopedPointer<GetDirentsRequest, QScopedPointerDeleteLater> get_dirents_req_;
