@@ -1,6 +1,7 @@
 #include <QtGlobal>
 #include <QtWidgets>
 #include <QDesktopServices>
+#include <QHBoxLayout>
 
 #include "seafile-applet.h"
 #include "account-mgr.h"
@@ -42,8 +43,8 @@ enum {
 };
 
 const char *kLoadingFailedLabelName = "LoadingFailedText";
-const int kToolBarIconSize = 20;
-const int kStatusBarIconSize = 24;
+const int kToolBarIconSize = 24;
+const int kStatusBarIconSize = 20;
 //const int kStatusCodePasswordNeeded = 400;
 
 void openFile(const QString& path)
@@ -67,6 +68,10 @@ bool inline findConflict(const QString &name, const QList<SeafDirent> &dirents)
     return found_conflict;
 }
 
+QString appendTrailingSlash(const QString& input) {
+    return input.endsWith('/') ? input : input + '/';
+}
+
 } // namespace
 
 QStringList FileBrowserDialog::file_names_to_be_pasted_;
@@ -192,6 +197,11 @@ FileBrowserDialog::FileBrowserDialog(const Account &account, const ServerRepo& r
     connect(data_mgr_, SIGNAL(removeDirentFailed(const ApiError&)),
             this, SLOT(onDirentRemoveFailed(const ApiError&)));
 
+    connect(data_mgr_, SIGNAL(removeDirentsSuccess(const QString&, const QStringList&)),
+            this, SLOT(onDirentsRemoveSuccess(const QString&, const QStringList&)));
+    connect(data_mgr_, SIGNAL(removeDirentsFailed(const ApiError&)),
+            this, SLOT(onDirentsRemoveFailed(const ApiError&)));
+
     //share <--> data_mgr_
     connect(data_mgr_, SIGNAL(shareDirentSuccess(const QString&)),
             this, SLOT(onDirentShareSuccess(const QString&)));
@@ -256,10 +266,17 @@ void FileBrowserDialog::createToolBar()
     toolbar_->addWidget(forward_button_);
     connect(forward_button_, SIGNAL(clicked()), this, SLOT(goForward()));
 
+    // XX: not sure why, but we have to set the styles here, otherwise it won't
+    // work (if we write this in qt.css)
+    backward_button_->setStyleSheet("QToolButton { margin-right: -2px; }");
+    forward_button_->setStyleSheet("QToolButton { margin-left: -2px; margin-right: 10px;}");
+    toolbar_->setStyleSheet("QToolbar { spacing: 0px; }");
+
     gohome_action_ = new QAction(tr("Home"), this);
     gohome_action_->setIcon(QIcon(":images/filebrowser/home.png"));
     connect(gohome_action_, SIGNAL(triggered()), this, SLOT(goHome()));
     toolbar_->addAction(gohome_action_);
+    toolbar_->widgetForAction(gohome_action_)->setObjectName("goHomeButton");
 
     path_navigator_ = new QButtonGroup(this);
     connect(path_navigator_, SIGNAL(buttonClicked(int)),
@@ -269,16 +286,16 @@ void FileBrowserDialog::createToolBar()
 
 void FileBrowserDialog::createStatusBar()
 {
-    status_bar_ = new QToolBar;
+    status_bar_ = new QWidget;
     status_bar_->setObjectName("statusBar");
-    status_bar_->setIconSize(QSize(kStatusBarIconSize, kStatusBarIconSize));
 
-    QWidget *spacer1 = new QWidget;
-    spacer1->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
-    status_bar_->addWidget(spacer1);
+    status_layout_ = new QHBoxLayout(status_bar_);
+    status_layout_->setContentsMargins(0, 0, 0, 0);
 
     // Submenu
-    upload_menu_ = new QMenu(status_bar_);
+    upload_menu_ = new QMenu;
+    connect(upload_menu_, SIGNAL(aboutToShow()), this, SLOT(fixUploadButtonHighlightStyle()));
+    connect(upload_menu_, SIGNAL(aboutToHide()), this, SLOT(fixUploadButtonNonHighlightStyle()));
 
     // Submenu's Action 1: Upload File
     upload_file_action_ = new QAction(tr("Upload files"), upload_menu_);
@@ -303,9 +320,11 @@ void FileBrowserDialog::createStatusBar()
     upload_button_ = new QToolButton;
     upload_button_->setObjectName("uploadButton");
     upload_button_->setToolTip(tr("Upload files"));
-    upload_button_->setIcon(QIcon(":/images/toolbar/add.png"));
+    upload_button_->setIcon(QIcon(":/images/filebrowser/add.png"));
+    upload_button_->setIconSize(QSize(kStatusBarIconSize, kStatusBarIconSize));
+    upload_button_->installEventFilter(this);
     connect(upload_button_, SIGNAL(clicked()), this, SLOT(uploadFileOrMkdir()));
-    status_bar_->addWidget(upload_button_);
+    status_layout_->addWidget(upload_button_);
 
     if (current_readonly_) {
         upload_button_->setEnabled(false);
@@ -315,18 +334,16 @@ void FileBrowserDialog::createStatusBar()
     details_label_ = new QLabel;
     details_label_->setAlignment(Qt::AlignCenter | Qt::AlignVCenter);
     details_label_->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
-    status_bar_->addWidget(details_label_);
+    status_layout_->addWidget(details_label_);
 
-    refresh_action_ = new QAction(this);
-    refresh_action_->setIcon(QIcon(":/images/toolbar/refresh.png"));
-    refresh_action_->setToolTip(tr("Refresh"));
-    connect(refresh_action_, SIGNAL(triggered()), this, SLOT(forceRefresh()));
-    refresh_action_->setShortcut(QKeySequence::Refresh);
-    status_bar_->addAction(refresh_action_);
-
-    QWidget *spacer2 = new QWidget;
-    spacer2->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
-    status_bar_->addWidget(spacer2);
+    refresh_button_ = new QToolButton;
+    refresh_button_->setObjectName("refreshButton");
+    refresh_button_->setToolTip(tr("Refresh"));
+    refresh_button_->setIcon(QIcon(":/images/filebrowser/refresh-gray.png"));
+    refresh_button_->setIconSize(QSize(kStatusBarIconSize, kStatusBarIconSize));
+    refresh_button_->installEventFilter(this);
+    connect(refresh_button_, SIGNAL(clicked()), this, SLOT(forceRefresh()));
+    status_layout_->addWidget(refresh_button_);
 }
 
 void FileBrowserDialog::createFileTable()
@@ -340,6 +357,35 @@ void FileBrowserDialog::createFileTable()
             this, SLOT(uploadOrUpdateMutipleFile(const QStringList&)));
 }
 
+bool FileBrowserDialog::eventFilter(QObject *obj, QEvent *event)
+{
+    if (obj == upload_button_) {
+        // Sometimes the upload menu is dismissed, the upload button style won't
+        // change, and we need to handle it manually here as well as in the
+        // aboutToHide signal of the upload_meu. This might be a qt bug.
+        if (event->type() == QEvent::Enter) {
+            fixUploadButtonHighlightStyle();
+            return true;
+        } else if (event->type() == QEvent::Leave) {
+            fixUploadButtonNonHighlightStyle();
+            return true;
+        }
+    } else if (obj == refresh_button_) {
+        if (event->type() == QEvent::Enter) {
+            refresh_button_->setStyleSheet("FileBrowserDialog QToolButton#refreshButton {"
+                                          "background: #DFDFDF; padding: 3px;"
+                                          "margin-right: 12px; border-radius: 2px;}");
+            return true;
+        } else if (event->type() == QEvent::Leave) {
+            refresh_button_->setStyleSheet("FileBrowserDialog QToolButton#refreshButton {"
+                                          "background: #F5F5F7; padding: 0px;"
+                                          "margin-right: 15px;}");
+            return true;
+        }
+    }
+    return QObject::eventFilter(obj, event);
+}
+
 void FileBrowserDialog::forceRefresh()
 {
     fetchDirents(true);
@@ -553,7 +599,7 @@ void FileBrowserDialog::enterPath(const QString& path)
         for(int i = 1; i < current_lpath_.size(); i++) {
             QLabel *separator = new QLabel("/");
             separator->setBaseSize(4, 7);
-            separator->setPixmap(QIcon(":/images/filebrowser/path-separator.png").pixmap(QSize(4, 7)));
+            separator->setPixmap(QIcon(":/images/filebrowser/path-separator.png").pixmap(10));
             path_navigator_separators_.push_back(separator);
             toolbar_->addWidget(separator);
             if (i != current_lpath_.size() - 1) {
@@ -628,7 +674,7 @@ void FileBrowserDialog::uploadFile(const QString& path, const QString& name,
 
     // set dialog attributes
     dialog->setAttribute(Qt::WA_DeleteOnClose);
-    dialog->setWindowModality(Qt::WindowModal);
+    dialog->setWindowModality(Qt::NonModal);
     dialog->show();
     dialog->raise();
     dialog->activateWindow();
@@ -639,27 +685,43 @@ void FileBrowserDialog::uploadMultipleFile(const QStringList& names,
 {
     if (names.empty())
         return;
-    const QString path = QFileInfo(names.front()).path();
+    QString local_path;
     QStringList fnames;
     Q_FOREACH(const QString &name, names) {
         const QFileInfo file = name;
         if (file.isDir()) {
+            // a dir
             uploadOrUpdateFile(name);
-        } else if (file.path() == path) {
-            fnames.push_back(file.fileName());
+        } else {
+            // a file
+            if (local_path.isEmpty()) {
+                local_path = file.path();
+                fnames.push_back(file.fileName());
+            } else if (file.path() == local_path) {
+                fnames.push_back(file.fileName());
+            } else {
+                qWarning(
+                    "upload multiple files: ignore file %s because it's not in "
+                    "the same directory",
+                    toCStr(file.absoluteFilePath()));
+            }
         }
     }
 
+    if (fnames.empty()) {
+        return;
+    }
+
     FileUploadTask *task =
-      data_mgr_->createUploadMultipleTask(repo_.id, current_path_, path, fnames,
-                                          overwrite);
+        data_mgr_->createUploadMultipleTask(repo_.id, current_path_, local_path, fnames,
+                                            overwrite);
     connect(task, SIGNAL(finished(bool)), this, SLOT(onUploadFinished(bool)));
     FileBrowserProgressDialog *dialog = new FileBrowserProgressDialog(task, this);
     task->start();
 
     // set dialog attributes
     dialog->setAttribute(Qt::WA_DeleteOnClose);
-    dialog->setWindowModality(Qt::WindowModal);
+    dialog->setWindowModality(Qt::NonModal);
     dialog->show();
     dialog->raise();
     dialog->activateWindow();
@@ -761,18 +823,7 @@ void FileBrowserDialog::onUploadFinished(bool success)
 
     // Upload Directory Task
     if (qobject_cast<FileUploadDirectoryTask *>(sender())) {
-        const SeafDirent dir = {
-          SeafDirent::DIR,
-          false,
-          "",
-          task->name(),
-          0,
-          QDateTime::currentDateTime().toTime_t(),
-          false,
-          false,
-          "",
-          0
-        };
+        const SeafDirent dir = SeafDirent::dir(task->name());
         // TODO: insert the Item prior to the item where uploading occurs
         table_model_->insertItem(0, dir);
         return;
@@ -801,18 +852,7 @@ void FileBrowserDialog::onUploadFinished(bool success)
     // add the items to tableview
     Q_FOREACH(const QString &name, names) {
         const QFileInfo file = QDir(local_path).filePath(name);
-        const SeafDirent dirent = {
-          SeafDirent::FILE,
-          false,
-          "",
-          name,
-          static_cast<quint64>(file.size()),
-          QDateTime::currentDateTime().toTime_t(),
-          false,
-          false,
-          "",
-          0
-        };
+        const SeafDirent dirent = SeafDirent::file(name, static_cast<quint64>(file.size()));
         if (task->useUpload())
             table_model_->appendItem(dirent);
         else
@@ -966,11 +1006,11 @@ void FileBrowserDialog::onGetDirentRemove(const QList<const SeafDirent*> &dirent
     if (!seafApplet->yesOrNoBox(tr("Do you really want to delete these items"), this, false))
         return;
 
-    Q_FOREACH(const SeafDirent* dirent, dirents)
-    {
-        data_mgr_->removeDirent(repo_.id, pathJoin(current_path_, dirent->name),
-                                dirent->isFile());
+    QStringList filenames;
+    foreach (const SeafDirent *dirent, dirents) {
+        filenames << dirent->name;
     }
+    data_mgr_->removeDirents(repo_.id, current_path_, filenames);
 }
 
 void FileBrowserDialog::onGetDirentShare(const SeafDirent& dirent)
@@ -1006,9 +1046,7 @@ void FileBrowserDialog::onDirectoryCreateSuccess(const QString &path)
     // if no longer current level
     if (::pathJoin(current_path_, name) != path)
         return;
-    const SeafDirent dirent = {SeafDirent::DIR, false, "", name, 0,
-                               QDateTime::currentDateTime().toTime_t(), false,
-                               false, "", 0};
+    const SeafDirent dirent = SeafDirent::dir(name);
     // TODO insert to the pos where the drop event triggered
     table_model_->insertItem(0, dirent);
 }
@@ -1037,6 +1075,7 @@ void FileBrowserDialog::onFileLockSuccess(const QString& path, bool lock)
     SeafDirent new_dirent = *old_dirent;
     new_dirent.is_locked = new_dirent.locked_by_me = lock;
     new_dirent.lock_owner = account_.username;
+    new_dirent.lock_owner_name = account_.accountInfo.name;
     table_model_->replaceItem(name, new_dirent);
 }
 
@@ -1083,6 +1122,23 @@ void FileBrowserDialog::onDirentRemoveFailed(const ApiError&error)
     seafApplet->warningBox(tr("Remove failed"), this);
 }
 
+void FileBrowserDialog::onDirentsRemoveSuccess(const QString& parent_path,
+                                               const QStringList& filenames)
+{
+    // if no longer current level
+    if (current_path_ != parent_path)
+        return;
+    foreach (const QString& name, filenames) {
+        // printf("removed file: %s\n", name.toUtf8().data());
+        table_model_->removeItemNamed(name);
+    }
+}
+
+void FileBrowserDialog::onDirentsRemoveFailed(const ApiError&error)
+{
+    seafApplet->warningBox(tr("Remove failed"), this);
+}
+
 void FileBrowserDialog::onDirentShareSuccess(const QString &link)
 {
     SharedLinkDialog(link, this).exec();
@@ -1128,11 +1184,27 @@ void FileBrowserDialog::setFilesToBePasted(bool is_copy, const QStringList &file
 
 void FileBrowserDialog::onGetDirentsPaste()
 {
-    if (repo_id_to_be_pasted_from_ == repo_.id &&
-        current_path_ == dir_path_to_be_pasted_from_) {
-        seafApplet->warningBox(tr("Cannot paste files from the same folder"), this);
-        return;
+    if (repo_id_to_be_pasted_from_ == repo_.id) {
+        if (current_path_ == dir_path_to_be_pasted_from_) {
+            seafApplet->warningBox(tr("Cannot paste files from the same folder"), this);
+            return;
+        }
+
+        if (file_names_to_be_pasted_.isEmpty()) {
+            return;
+        }
+
+        // Paste /a/ into /a/b/ is not allowed
+        for (const QString& name : file_names_to_be_pasted_) {
+            const QString file_path_to_be_pasted =
+                appendTrailingSlash(::pathJoin(dir_path_to_be_pasted_from_, name));
+            if (appendTrailingSlash(current_path_).startsWith(file_path_to_be_pasted)) {
+                seafApplet->warningBox(tr("Cannot paste the folder to its subfolder"), this);
+                return;
+            }
+        }
     }
+
     if (is_copyed_when_pasted_)
         data_mgr_->copyDirents(repo_id_to_be_pasted_from_,
                                dir_path_to_be_pasted_from_,
@@ -1199,3 +1271,31 @@ void FileBrowserDialog::onCreateSubrepoFailed(const ApiError&error)
 {
     seafApplet->warningBox(tr("Create library failed!"), this);
 }
+
+void FileBrowserDialog::fixUploadButtonHighlightStyle()
+{
+    fixUploadButtonStyle(true);
+}
+
+void FileBrowserDialog::fixUploadButtonNonHighlightStyle()
+{
+    fixUploadButtonStyle(false);
+}
+
+void FileBrowserDialog::fixUploadButtonStyle(bool highlighted)
+{
+    if (highlighted) {
+        upload_button_->setStyleSheet("FileBrowserDialog QToolButton#uploadButton {"
+                                      "background: #DFDFDF; padding: 3px;"
+                                      "margin-left: 12px; border-radius: 2px;}");
+    } else {
+        // XX: underMouse() return true even if the upload button is not under mouse!
+        //
+        // if (upload_button_->underMouse()) {
+        //     return;
+        // }
+        upload_button_->setStyleSheet("FileBrowserDialog QToolButton#uploadButton {"
+                                      "background: #F5F5F7; padding: 0px;"
+                                      "margin-left: 15px;}");
+    }
+}
