@@ -27,6 +27,7 @@ enum {
     FILE_MAX_COLUMN
 };
 
+int kMarginLeft = 9;
 const int kDefaultColumnWidth = 120;
 const int kDefaultColumnHeight = 40;
 const int kColumnIconSize = 28;
@@ -39,11 +40,12 @@ const int kRefreshProgressInterval = 1000;
 
 const QColor kSelectedItemBackgroundcColor("#F9E0C7");
 const QColor kItemBackgroundColor("white");
-const QColor kItemBottomBorderColor("#f3f3f3");
-const QColor kItemColor("black");
+const QColor kItemBottomBorderColor("#ECECEC");
+const QColor kFileNameFontColor("black");
+const QColor kFontColor("#757575");
 const QString kProgressBarStyle("QProgressBar "
         "{ border: 1px solid grey; border-radius: 2px; } "
-        "QProgressBar::chunk { background-color: #f0f0f0; width: 20px; }");
+        "QProgressBar::chunk { background-color: #f0f0f0; width: 1px; }");
 
 enum {
     NOT_LOCKED = 0,
@@ -66,14 +68,6 @@ void FileTableViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem
 
     // fix for the last item
     QRect option_rect = option.rect;
-    if (index.column() == FILE_COLUMN_KIND) {
-        option_rect.setSize(option_rect.size() - QSize(13, 0));
-        // white the extra rect
-        const QRect last_rect = QRect(option_rect.topRight(), QSize(13, option_rect.height()));
-        painter->save();
-        painter->fillRect(last_rect, kItemBackgroundColor);
-        painter->restore();
-    }
 
     // draw item's background
     painter->save();
@@ -112,7 +106,7 @@ void FileTableViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem
     {
         // draw icon
         QPixmap pixmap = model->data(index, Qt::DecorationRole).value<QPixmap>();
-        int scale_factor = devicePixelRatio();
+        double scale_factor = globalDevicePixelRatio();
         // On Mac OSX (and other HDPI screens) the pixmap would be the 2x
         // version (but the draw rect area is still the same size), so when
         // computing the offsets we need to divide it by the scale factor.
@@ -123,8 +117,12 @@ void FileTableViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem
         int alignX = (kColumnIconSize - icon_width) / 2;
         int alignY = (size.height() - icon_height) / 2;
 
+#ifdef Q_OS_WIN32
+    kMarginLeft = 4;
+#endif
+
         QRect icon_bound_rect(
-            option_rect.topLeft() + QPoint(alignX, alignY - 2),
+            option_rect.topLeft() + QPoint(kMarginLeft + alignX, alignY - 2),
             QSize(icon_width, icon_height));
 
         painter->save();
@@ -139,14 +137,15 @@ void FileTableViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem
             QPixmap locked_pixmap = QIcon(image).pixmap(kLockIconSize, kLockIconSize);
             int alignX = (kColumnIconSize / 2) + 3;
             int alignY = (kColumnIconSize / 2) + 2;
-            painter->drawPixmap(option_rect.topLeft() + QPoint(alignX, alignY), locked_pixmap);
+            painter->drawPixmap(option_rect.topLeft() + QPoint(kMarginLeft + alignX, alignY), locked_pixmap);
             painter->restore();
         }
 
         // draw text
         QFont font = model->data(index, Qt::FontRole).value<QFont>();
-        QRect rect(option_rect.topLeft() + QPoint(4 * 2 + kColumnIconSize, -2), size - QSize(kColumnIconSize, 0));
-        painter->setPen(kItemColor);
+        QRect rect(option_rect.topLeft() + QPoint(kMarginLeft + 2 * 2 + kColumnIconSize, -2),
+                   size - QSize(kColumnIconSize + kMarginLeft, 0));
+        painter->setPen(kFileNameFontColor);
         painter->setFont(font);
         painter->drawText(rect,
                           Qt::AlignLeft | Qt::AlignVCenter | Qt::TextSingleLine,
@@ -193,9 +192,9 @@ void FileTableViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem
             text = model->data(index, Qt::UserRole).toString();
         }
         QFont font = model->data(index, Qt::FontRole).value<QFont>();
-        QRect rect(option_rect.topLeft() + QPoint(4, -2), size - QSize(10, 0));
+        QRect rect(option_rect.topLeft() + QPoint(9, -2), size - QSize(10, 0));
         painter->save();
-        painter->setPen(kItemColor);
+        painter->setPen(kFontColor);
         painter->setFont(font);
         painter->drawText(rect, Qt::AlignLeft | Qt::AlignVCenter | Qt::TextSingleLine, text);
         painter->restore();
@@ -219,7 +218,7 @@ FileTableView::FileTableView(QWidget *parent)
       proxy_model_(NULL)
 {
     verticalHeader()->hide();
-    verticalHeader()->setDefaultSectionSize(36);
+    verticalHeader()->setDefaultSectionSize(kDefaultColumnHeight);
     horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
     horizontalHeader()->setStretchLastSection(true);
     horizontalHeader()->setCascadingSectionResizes(true);
@@ -289,10 +288,6 @@ void FileTableView::setupContextMenu()
     context_menu_ = new QMenu(this);
     connect(parent_, SIGNAL(aboutToClose()),
             context_menu_, SLOT(close()));
-    download_action_ = new QAction(tr("&Open"), this);
-    connect(download_action_, SIGNAL(triggered()),
-            this, SLOT(onOpen()));
-    download_action_->setShortcut(QKeySequence::InsertParagraphSeparator);
 
     saveas_action_ = new QAction(tr("&Save As..."), this);
     connect(saveas_action_, SIGNAL(triggered()),
@@ -315,7 +310,7 @@ void FileTableView::setupContextMenu()
     remove_action_ = new QAction(tr("&Delete"), this);
     connect(remove_action_, SIGNAL(triggered()),
             this, SLOT(onRemove()));
-    remove_action_->setShortcut(QKeySequence::Delete);
+    // remove_action_->setShortcut(QKeySequence::Delete);
 
     share_action_ = new QAction(tr("&Generate %1 Download Link").arg(getBrand()), this);
     connect(share_action_, SIGNAL(triggered()),
@@ -368,14 +363,12 @@ void FileTableView::setupContextMenu()
     sync_subdirectory_action_ = new QAction(tr("&Sync this folder"), this);
     connect(sync_subdirectory_action_, SIGNAL(triggered()),
             this, SLOT(onSyncSubdirectory()));
-    sync_subdirectory_action_->setShortcut(Qt::ALT + Qt::Key_S);
+    // sync_subdirectory_action_->setShortcut(Qt::ALT + Qt::Key_S);
     if (!parent_->account_.isAtLeastVersion(4, 1, 0)) {
         sync_subdirectory_action_->setEnabled(false);
         sync_subdirectory_action_->setToolTip(tr("This feature is available in pro version only\n"));
     }
 
-    context_menu_->setDefaultAction(download_action_);
-    context_menu_->addAction(download_action_);
     context_menu_->addAction(saveas_action_);
     context_menu_->addAction(share_action_);
     context_menu_->addAction(share_seafile_action_);
@@ -390,11 +383,10 @@ void FileTableView::setupContextMenu()
     context_menu_->addAction(rename_action_);
     context_menu_->addAction(remove_action_);
     context_menu_->addSeparator();
-    context_menu_->addAction(update_action_);
+    // context_menu_->addAction(update_action_);
     context_menu_->addAction(cancel_download_action_);
     context_menu_->addAction(sync_subdirectory_action_);
 
-    this->addAction(download_action_);
     this->addAction(saveas_action_);
     this->addAction(share_action_);
     this->addAction(share_seafile_action_);
@@ -404,7 +396,7 @@ void FileTableView::setupContextMenu()
     this->addAction(lock_action_);
     this->addAction(rename_action_);
     this->addAction(remove_action_);
-    this->addAction(update_action_);
+    // this->addAction(update_action_);
     this->addAction(cancel_download_action_);
     this->addAction(sync_subdirectory_action_);
 
@@ -418,6 +410,8 @@ void FileTableView::contextMenuEvent(QContextMenuEvent *event)
     const QModelIndex proxy_index = indexAt(position);
     position = viewport()->mapToGlobal(position);
 
+    paste_action_->setVisible(parent_->hasFilesToBePasted());
+
     //
     // show paste only menu for no items
     // paste-dedicated menu
@@ -435,7 +429,10 @@ void FileTableView::contextMenuEvent(QContextMenuEvent *event)
     //
     paste_action_->setVisible(parent_->hasFilesToBePasted());
     paste_action_->setEnabled(!parent_->current_readonly_ &&
-        parent_->account_to_be_pasted_from_ == parent_->account_);
+                              parent_->account_to_be_pasted_from_ ==
+                                  parent_->account_ &&
+                              parent_->hasFilesToBePasted());
+    // printf ("paste action enabled: %s\n", paste_action_->isEnabled() ? "true": "false");
 
     //
     // map back to the source index from FileTableModel
@@ -476,17 +473,16 @@ void FileTableView::contextMenuEvent(QContextMenuEvent *event)
             }
         }
 
-        download_action_->setVisible(!has_dir);
         saveas_action_->setText(tr("&Save As To..."));
+        saveas_action_->setVisible(true);
         saveas_action_->setEnabled(!has_dir);
-        download_action_->setText(tr("D&ownload"));
         lock_action_->setVisible(false);
         rename_action_->setVisible(false);
         share_action_->setVisible(false);
         share_seafile_action_->setVisible(false);
         share_to_user_action_->setVisible(false);
         share_to_group_action_->setVisible(false);
-        update_action_->setVisible(false);
+        // update_action_->setVisible(false);
         cancel_download_action_->setVisible(false);
         sync_subdirectory_action_->setVisible(false);
 
@@ -505,30 +501,28 @@ void FileTableView::contextMenuEvent(QContextMenuEvent *event)
     item_.reset(new SeafDirent(*dirent));
 
     saveas_action_->setText(tr("&Save As..."));
+    saveas_action_->setVisible(true);
     rename_action_->setVisible(true);
     share_action_->setVisible(true);
     share_seafile_action_->setVisible(true);
-    update_action_->setVisible(true);
+    // update_action_->setVisible(true);
     cancel_download_action_->setVisible(true);
-    download_action_->setVisible(true);
-    download_action_->setEnabled(true);
     cancel_download_action_->setVisible(false);
     if (item_->readonly) {
         move_action_->setEnabled(false);
         rename_action_->setEnabled(false);
-        update_action_->setEnabled(false);
+        // update_action_->setEnabled(false);
         remove_action_->setEnabled(false);
     } else {
         move_action_->setEnabled(true);
         rename_action_->setEnabled(true);
-        update_action_->setEnabled(true);
+        // update_action_->setEnabled(true);
         remove_action_->setEnabled(true);
     }
 
     if (item_->isDir()) {
         lock_action_->setVisible(false);
-        update_action_->setVisible(false);
-        download_action_->setText(tr("&Open"));
+        // update_action_->setVisible(false);
         saveas_action_->setEnabled(false);
         sync_subdirectory_action_->setVisible(true);
         share_to_user_action_->setVisible(true);
@@ -544,8 +538,7 @@ void FileTableView::contextMenuEvent(QContextMenuEvent *event)
             lock_action_->setVisible(true);
         }
 
-        update_action_->setVisible(true);
-        download_action_->setText(tr("D&ownload"));
+        // update_action_->setVisible(true);
         saveas_action_->setEnabled(true);
         sync_subdirectory_action_->setVisible(false);
         share_to_user_action_->setVisible(false);
@@ -554,7 +547,6 @@ void FileTableView::contextMenuEvent(QContextMenuEvent *event)
         if (TransferManager::instance()->getDownloadTask(parent_->repo_.id,
             ::pathJoin(parent_->current_path_, dirent->name))) {
             cancel_download_action_->setVisible(true);
-            download_action_->setVisible(false);
             saveas_action_->setVisible(false);
         }
     }
@@ -589,30 +581,6 @@ void FileTableView::onItemDoubleClicked(const QModelIndex& index)
     emit direntClicked(*dirent);
 }
 
-void FileTableView::onOpen()
-{
-    if (item_ == NULL) {
-        const QList<const SeafDirent*> dirents = getSelectedItemsFromSource();
-        // if we are going to open a directory
-        if (dirents.size() == 1 &&
-            dirents.front()->isDir()) {
-            emit direntClicked(*dirents.front());
-            return;
-        }
-        // in other cases, download files only
-        for (int i = 0; i < dirents.size(); i++) {
-            // ignore directories since we have no support for them
-            if (dirents[i]->isDir())
-                continue;
-
-            emit direntClicked(*dirents[i]);
-        }
-        return;
-    }
-
-    emit direntClicked(*item_);
-}
-
 void FileTableView::onSaveAs()
 {
     if (item_ == NULL) {
@@ -911,7 +879,7 @@ QVariant FileTableModel::data(const QModelIndex & index, int role) const
                 dialog->repo_.id,
                 ::pathJoin(dialog->current_path_, dirent.name),
                 dirent.id,
-                kColumnIconSize * devicePixelRatio());
+                kColumnIconSize * globalDevicePixelRatio());
         } else {
             icon = QIcon(getIconByFileNameV2(dirent.name));
         }
@@ -952,7 +920,7 @@ QVariant FileTableModel::data(const QModelIndex & index, int role) const
     }
 
     if (role == Qt::ToolTipRole && column == FILE_COLUMN_NAME && !dirent.lock_owner.isEmpty()) {
-        return tr("locked by %1").arg(dirent.lock_owner);
+        return tr("locked by %1").arg(dirent.getLockOwnerDisplayString());
     }
 
     if (role != Qt::DisplayRole) {
@@ -989,27 +957,46 @@ QVariant FileTableModel::headerData(int section,
                                     Qt::Orientation orientation,
                                     int role) const
 {
-    if (orientation == Qt::Vertical)
-        return QVariant();
-
-    if (role == Qt::TextAlignmentRole)
-        return Qt::AlignLeft + Qt::AlignVCenter;
-
-    if (role != Qt::DisplayRole)
-        return QVariant();
-
-    switch (section) {
-    case FILE_COLUMN_NAME:
-        return tr("Name");
-    case FILE_COLUMN_SIZE:
-        return tr("Size");
-    case FILE_COLUMN_MTIME:
-        return tr("Last Modified");
-    case FILE_COLUMN_KIND:
-        return tr("Kind");
-    default:
+    if (orientation == Qt::Vertical) {
         return QVariant();
     }
+
+    if (role == Qt::TextAlignmentRole) {
+        return Qt::AlignLeft + Qt::AlignVCenter;
+    }
+
+    if (role == Qt::DisplayRole) {
+        switch (section) {
+        case FILE_COLUMN_NAME:
+            return tr("Name");
+        case FILE_COLUMN_SIZE:
+            return tr("Size");
+        case FILE_COLUMN_MTIME:
+            return tr("Last Modified");
+        case FILE_COLUMN_KIND:
+            return tr("Kind");
+        default:
+            return QVariant();
+        }
+    }
+
+    if (role == Qt::FontRole) {
+        QFont font;
+        font.setPixelSize(12);
+        return font;
+    }
+
+    if (role == Qt::ForegroundRole) {
+        return QBrush(kFontColor);
+    }
+
+    if (role == Qt::SizeHintRole && section == FILE_COLUMN_NAME) {
+        if (dirents_.empty()) {
+            return QSize(name_column_width_, 0);
+        }
+    }
+
+    return QVariant();
 }
 
 const SeafDirent* FileTableModel::direntAt(int row) const
