@@ -16,10 +16,13 @@ class QNetworkAccessManager;
 class QNetworkReply;
 class QThread;
 class QSslError;
+class QNetworkRequest;
 
 class FileServerTask;
 class ApiError;
 
+class ReliablePostFileTask;
+
 template<typename T> class QList;
 
 // deleter
@@ -48,6 +51,7 @@ struct doDeleteLater
  */
 class FileNetworkTask: public QObject {
     Q_OBJECT
+    friend class ReliablePostFileTask;
 public:
     enum TaskType {
         Upload,
@@ -76,6 +80,7 @@ public:
     QString fileName() const;
     QString oid() const { return oid_; }
     Progress progress() const { return progress_; };
+    bool canceled() const { return canceled_; }
 
     enum TaskError {
         NoError = 0,
@@ -107,6 +112,7 @@ public slots:
 signals:
     void progressUpdate(qint64 transferred, qint64 total);
     void finished(bool success);
+    void retried(int retry_count);
 
 protected slots:
     void onFileServerTaskProgressUpdate(qint64 transferred, qint64 total);
@@ -268,13 +274,15 @@ public:
     const FileNetworkTask::TaskError& error() const { return error_; }
     const QString& errorString() const { return error_string_; }
     int httpErrorCode() const { return http_error_code_; }
-    const QString& oid() const { return oid_; }
+    virtual const QString& oid() const { return oid_; }
+    int retryCount() const { return retry_count_; }
 
     static void resetQNAM();
 
 signals:
     void progressUpdate(qint64 transferred, qint64 total);
     void finished(bool success);
+    void retried(int retry_count);
 
 public slots:
     void start();
@@ -283,14 +291,22 @@ public slots:
 protected slots:
     void onSslErrors(const QList<QSslError>& errors);
     void httpRequestFinished();
+    void retry();
 
 protected:
     /**
-     * Prepare the initialization work, like creating the tmp file, or open
-     * the uploaded file for reading.
+     * Prepare the initialization work, like creating the tmp file, or open the
+     * uploaded file for reading. It may be called again when retrying a task.
      */
     virtual void prepare() = 0;
 
+    /**
+     * If return true, the request would be retried for a few times when error
+     * happened. By default the request is not retried. Subclasses can overwrite
+     * this function to enable retry.
+     */
+    virtual bool retryEnabled();
+
     /**
      * Send the http request.
      * This member function may be called in two places:
@@ -300,13 +316,10 @@ protected:
     virtual void sendRequest() = 0;
     virtual void onHttpRequestFinished() = 0;
     bool handleHttpRedirect();
+    bool maybeRetry();
     void setError(FileNetworkTask::TaskError error, const QString& error_string);
     void setHttpError(int code);
 
-    static bool should_reset_qnam_;
-    static QMutex network_mgr_lock_;
-    static QNetworkAccessManager *network_mgr_;
-
     // Always use this to access the network access manager, instead of using
     // network_mgr_ directly, because it handles the network status change
     // detection.
@@ -319,6 +332,7 @@ protected:
     QNetworkReply *reply_;
     bool canceled_;
     int redirect_count_;
+    int retry_count_;
 
     FileNetworkTask::TaskError error_;
     QString error_string_;
@@ -343,35 +357,6 @@ private:
     QTemporaryFile *tmp_file_;
 };
 
-class PostFileTask : public FileServerTask {
-    Q_OBJECT
-public:
-    PostFileTask(const QUrl& url,
-                 const QString& parent_dir,
-                 const QString& local_path,
-                 const QString& name,
-                 const bool use_upload);
-
-    PostFileTask(const QUrl& url,
-                 const QString& parent_dir,
-                 const QString& local_path,
-                 const QString& name,
-                 const QString& relative_path);
-    ~PostFileTask();
-
-protected:
-    void prepare();
-    void sendRequest();
-    void onHttpRequestFinished();
-
-private:
-    const QString parent_dir_;
-    QFile *file_;
-    const QString name_;
-    const bool use_upload_;
-    const QString relative_path_;
-};
-
 class PostFilesTask : public FileServerTask {
     Q_OBJECT
 public:
@@ -403,7 +388,7 @@ private:
     QList<qint64> file_sizes_;
     const QStringList names_;
 
-    QScopedPointer<PostFileTask, doDeleteLater<PostFileTask> > task_;
+    QScopedPointer<ReliablePostFileTask, doDeleteLater<ReliablePostFileTask> > task_;
     int current_num_;
     // transferred bytes in the current tasks
     qint64 current_bytes_;
