@@ -2,12 +2,15 @@
 #define SEAFILE_CLIENT_UI_ACCOUNT_VIEW_H
 
 #include <QWidget>
+
+#include "utils/singleton.h"
 #include "ui_account-view.h"
 
 class Account;
 class QAction;
 class QMenu;
 class ApiError;
+class QLabel;
 
 /*
  * The account information area, right below the header
@@ -47,6 +50,9 @@ private:
     QAction *add_account_action_;
     QAction *account_settings_action_;
     QMenu *account_menu_;
+
+signals:
+    void refresh();
 };
 
 #endif // SEAFILE_CLIENT_UI_ACCOUNT_VIEW_H
