@@ -336,52 +336,77 @@ void LoginDialog::showWarning(const QString& msg)
 }
 
 #ifdef HAVE_SHIBBOLETH_SUPPORT
-void LoginDialog::loginWithShib()
-{
-    QString serverAddr =
-        seafApplet->readPreconfigureEntry(kPreconfigureShibbolethLoginUrl)
-            .toString()
-            .trimmed();
-    if (!serverAddr.isEmpty()) {
-        if (QUrl(serverAddr).isValid()) {
-            qWarning("Using preconfigured shibboleth login url: %s\n",
-                     toCStr(serverAddr));
-        } else {
-            qWarning("Invalid preconfigured shibboleth login url: %s\n",
-                     toCStr(serverAddr));
-            serverAddr = "";
-        }
-    }
 
-    if (serverAddr.isEmpty()) {
-        // When we reach here, there is no preconfigured shibboleth login url,
-        // or the preconfigured url is invalid. So we ask the user for the url.
-        QString lastUsedShibUrl = seafApplet->settingsManager()->getLastShibUrl();
-        serverAddr =
+bool LoginDialog::getShibLoginUrl(const QString& last_shib_url, QUrl *url_out)
+{
+    QString server_addr = last_shib_url;
+    QUrl url;
+
+    while (true) {
+        bool ok;
+        server_addr =
             seafApplet->getText(this,
                                 tr("Shibboleth Login"),
                                 tr("%1 Server Address").arg(getBrand()),
                                 QLineEdit::Normal,
-                                lastUsedShibUrl);
-        serverAddr = serverAddr.trimmed();
+                                server_addr,
+                                &ok);
+        server_addr = server_addr.trimmed();
+
+        // exit when user hits cancel button
+        if (!ok) {
+            return false;
+        }
+
+        if (server_addr.isEmpty()) {
+            showWarning(tr("Server address must not be empty").arg(server_addr));
+            continue;
+        }
+
+        if (!server_addr.startsWith("https://")) {
+            showWarning(tr("%1 is not a valid server address. It has to start with 'https://'").arg(server_addr));
+            continue;
+        }
+
+        url = QUrl(server_addr, QUrl::StrictMode);
+        if (!url.isValid()) {
+            showWarning(tr("%1 is not a valid server address").arg(server_addr));
+            continue;
+        }
+
+        *url_out = url;
+        return true;
+    }
+}
+
+void LoginDialog::loginWithShib()
+{
+    QString server_addr =
+        seafApplet->readPreconfigureEntry(kPreconfigureShibbolethLoginUrl)
+            .toString()
+            .trimmed();
+    if (!server_addr.isEmpty()) {
+        if (QUrl(server_addr).isValid()) {
+            qWarning("Using preconfigured shibboleth login url: %s\n",
+                     toCStr(server_addr));
+        } else {
+            qWarning("Invalid preconfigured shibboleth login url: %s\n",
+                     toCStr(server_addr));
+            server_addr = "";
+        }
     }
 
-    if (serverAddr.isEmpty()) {
-        return;
+    QUrl url;
+    if (server_addr.isEmpty()) {
+        // When we reach here, there is no preconfigured shibboleth login url,
+        // or the preconfigured url is invalid. So we ask the user for the url.
+        server_addr = seafApplet->settingsManager()->getLastShibUrl();
+        if (!getShibLoginUrl(server_addr, &url)) {
+            return;
+        }
     }
 
-    if (!serverAddr.startsWith("http://") && !serverAddr.startsWith("https://")) {
-        showWarning(tr("%1 is not a valid server address").arg(serverAddr));
-        return;
-    }
-
-    QUrl url = QUrl(serverAddr, QUrl::StrictMode);
-    if (!url.isValid()) {
-        showWarning(tr("%1 is not a valid server address").arg(serverAddr));
-        return;
-    }
-
-    seafApplet->settingsManager()->setLastShibUrl(serverAddr);
+    seafApplet->settingsManager()->setLastShibUrl(url.toString());
 
     ShibLoginDialog shib_dialog(url, mComputerName->text(), this);
     if (shib_dialog.exec() == QDialog::Accepted) {
