@@ -28,20 +28,28 @@ namespace {
             subtitle
  */
 
-const int kMarginLeft = 5;
+const int kMarginLeft = 16;
 const int kMarginRight = 5;
 const int kMarginTop = 5;
 const int kMarginBottom = 5;
 const int kPadding = 5;
+const int kRepoItemVerticalMargin = 13;
+const int kRepoCategoryVerticalMargin = 10;
 
 const int kRepoIconHeight = 36;
 const int kRepoIconWidth = 36;
+const int kRepoIconHeightAlpha = 24;
+const int kRepoIconWidthAlpha = 24;
 const int kRepoNameWidth = 175;
 const int kRepoNameHeight = 30;
+const int kRepoNameHeightAlpha = 20;
+const int kRepoDescriptionHeightAlpha = 14;
 const int kRepoStatusIconWidth = 24;
 const int kRepoStatusIconHeight = 24;
+const int kRepoStatusIconWidthAlpha = 16;
+const int kRepoStatusIconHeightAlpha = 16;
 
-const char *kRepoCategoryIndicatorColor = "#AAA";
+const char *kRepoCategoryIndicatorColor = "#BBB";
 const int kRepoCategoryNameMaxWidth = 400;
 const int kRepoCategoryIndicatorWidth = 16;
 const int kRepoCategoryIndicatorHeight = 16;
@@ -50,28 +58,30 @@ const int kMarginBetweenIndicatorAndName = 5;
 const int kMarginBetweenRepoIconAndName = 10;
 const int kMarginBetweenRepoNameAndStatus = 20;
 
-const char *kRepoNameColor = "#3F3F3F";
+const char *kRepoNameColor = "#141414";
 const char *kRepoNameColorHighlighted = "#544D49";
-const char *kTimestampColor = "#959595";
+const char *kTimestampColor = "#AAAAAA";
 const char *kTimestampColorHighlighted = "#9D9B9A";
 const int kRepoNameFontSize = 14;
 const int kTimestampFontSize = 12;
+const int kRepoDescriptionFontSizeAlpha = 10;
 const int kRepoCategoryNameFontSize = 14;
 const int kRepoCategoryCountFontSize = 12;
+const int kRepoCategoryCountFontSizeAlpha = 14;
 const int kOwnerFontSize = 12;
+const int kOwnerFontSizeAlpha = 10;
 
-const char *kRepoItemBackgroundColorHighlighted = "#F9E0C7";
+const char *kRepoItemBackgroundColorHighlightedAlpha = "#EFEEEE";
 const char *kRepoItemBackgroundColor = "white";
-const char *kRepoItemBackgroundColorDragMove = "#C8C8C8";
+const char *kRepoItemBackgroundColorDragMove = "#EFEEEE";
 
-const char *kRepoCategoryColor = "#3F3F3F";
-//const char *kRepoCategoryColorHighlighted = "#FAF5FB";
+const char *kRepoCategoryColor = "#747474";
 
 const char *kRepoCategoryBackgroundColor = "white";
-//const char *kRepoCategoryBackgroundColorHighlighted = "#EF7544";
 
 const int kRepoCategoryCountMarginRight = 10;
-const char *kRepoCategoryCountColor = "#BBB";
+const int kRepoCategoryCountMarginRightAlpha = 20;
+const char *kRepoCategoryCountColor = "#AAAAAA";
 
 static void showTooltip(const QString &text,
                         QWidget *viewport,
@@ -111,12 +121,13 @@ QSize RepoItemDelegate::sizeHintForRepoItem(const QStyleOptionViewItem &option,
                                             const RepoItem *item) const
 {
 
-    int width = kMarginLeft + kRepoIconWidth
+    int width = kMarginLeft + kRepoIconWidthAlpha
         + kMarginBetweenRepoIconAndName + kRepoNameWidth
-        + kMarginBetweenRepoNameAndStatus + kRepoStatusIconWidth
+        + kMarginBetweenRepoNameAndStatus + kRepoStatusIconWidthAlpha
         + kMarginRight + kPadding * 2;
 
-    int height = kRepoIconHeight + kPadding * 2 + kMarginTop + kMarginBottom;
+    // int height = kRepoIconHeightAlpha + kPadding * 2 + kMarginTop + kMarginBottom;
+    int height = kRepoIconHeightAlpha + kRepoItemVerticalMargin * 2;
 
     // qDebug("width = %d, height = %d\n", width, height);
 
@@ -128,13 +139,17 @@ QSize RepoItemDelegate::sizeHintForRepoCategoryItem(const QStyleOptionViewItem &
 {
     int width, height;
 
-	QFontMetrics qfm(option.font);
+    QFontMetrics qfm(changeFontSize(option.font, kRepoCategoryNameFontSize));
     QSize size = qfm.size(0, item->name());
 
     width = qMin(size.width(), kRepoCategoryIndicatorWidth)
         + kMarginBetweenIndicatorAndName + kRepoCategoryNameMaxWidth;
 
-    height = qMax(size.height(), kRepoCategoryIndicatorHeight) + kPadding;
+    // height = qMax(size.height(), kRepoCategoryIndicatorHeight) + kPadding;
+    height = qMax(size.height(), kRepoCategoryIndicatorHeight) + kRepoCategoryVerticalMargin * 2;
+
+    RepoTreeModel *model = (RepoTreeModel *)item->model();
+    model->repo_category_height = height;
 
     // qDebug("width = %d, height = %d\n", width, height);
 
@@ -160,11 +175,15 @@ void RepoItemDelegate::paint(QPainter *painter,
 
     // fix for showing first category if hidden
     RepoTreeView *view = static_cast<RepoTreeView*>(parent());
-    if (view->indexAt(QPoint(0, 0)).parent().isValid()) {
+    QModelIndex top_index = view->indexAt(QPoint(0, 0)).parent();
+    if (top_index.isValid()) {
         if (option.rect.contains(QPoint(0, 0))) {
+            const RepoCategoryItem *category_item =
+                static_cast<RepoCategoryItem *>(getItem(top_index));
             QStyleOptionViewItem category_option = option;
-            category_option.rect = QRect(0, 0, option.rect.width(), kRepoCategoryIndicatorHeight + kPadding * 2);
-            paintRepoCategoryItem(painter, category_option, (RepoCategoryItem *)getItem(view->indexAt(QPoint(0, 0)).parent()));
+            category_option.rect = QRect(0, 0, option.rect.width(),
+                                         sizeHintForRepoCategoryItem(option, category_item).height());
+            paintRepoCategoryItem(painter, category_option, category_item);
         }
     }
 }
@@ -181,7 +200,7 @@ void RepoItemDelegate::paintRepoItem(QPainter *painter,
     RepoTreeView *view = model->treeView();
     QModelIndex index = ((QSortFilterProxyModel *)view->model())->mapFromSource(model->indexFromItem(item));
     if (option.state & (QStyle::State_HasFocus | QStyle::State_Selected)) {
-        backBrush = QColor(kRepoItemBackgroundColorHighlighted);
+        backBrush = QColor(kRepoItemBackgroundColorHighlightedAlpha);
         selected = true;
     } else if (view->getCurrentDropTarget() == index) {
         backBrush = QColor(kRepoItemBackgroundColorDragMove);
@@ -199,32 +218,38 @@ void RepoItemDelegate::paintRepoItem(QPainter *painter,
     }
 
     // Paint repo icon
-    QPoint repo_icon_pos(kMarginLeft + kPadding + indent_left, kMarginTop + kPadding);
+    // QPoint repo_icon_pos(kMarginLeft + kPadding + indent_left, kMarginTop + kPadding);
+    QPoint repo_icon_pos(kMarginLeft + 4 + indent_left, kRepoItemVerticalMargin);
     repo_icon_pos += option.rect.topLeft();
-    painter->save();
 
     // get the device pixel radio from current painter device
     int scale_factor = 1;
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
     scale_factor = painter->device()->devicePixelRatio();
 #endif // QT5
-    QPixmap repo_icon(repo.getIcon().pixmap(QSize(kRepoIconWidth, kRepoIconHeight) * scale_factor));
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
-    repo_icon.setDevicePixelRatio(scale_factor);
-#endif // QT5
+    QPixmap repo_icon = repo.getIcon().pixmap(QSize(kRepoIconWidthAlpha, kRepoIconHeightAlpha));
 
-    QRect repo_icon_rect(repo_icon_pos, QSize(kRepoIconWidth, kRepoIconHeight));
+    QRect repo_icon_rect(repo_icon_pos, QSize(kRepoIconWidthAlpha, kRepoIconHeightAlpha));
+    painter->save();
     painter->drawPixmap(repo_icon_rect, repo_icon);
     painter->restore();
 
     // Paint repo name
-    painter->save();
-    QPoint repo_name_pos = repo_icon_pos + QPoint(kRepoIconWidth + kMarginBetweenRepoIconAndName, 0);
-    int repo_name_width = option.rect.width() - kRepoIconWidth - kMarginBetweenRepoIconAndName
-        - kRepoStatusIconWidth  - kMarginBetweenRepoNameAndStatus
+    int vertical_margin_between_icon_and_name = 0;
+#ifdef Q_OS_WIN32
+    vertical_margin_between_icon_and_name = -9;
+#else
+    vertical_margin_between_icon_and_name = -5;
+#endif
+    QPoint repo_name_pos = repo_icon_pos + QPoint(kRepoIconWidthAlpha + kMarginBetweenRepoIconAndName,
+                                                  vertical_margin_between_icon_and_name);
+    int repo_name_width = option.rect.width() - kRepoIconWidthAlpha - kMarginBetweenRepoIconAndName
+        - kRepoStatusIconWidthAlpha - kMarginBetweenRepoNameAndStatus
         - kPadding * 2 - kMarginLeft - kMarginRight;
     repo_name_width -= indent_left;
-    QRect repo_name_rect(repo_name_pos, QSize(repo_name_width, kRepoNameHeight));
+    int repo_name_height = ::textHeightInFont(repo.name, changeFontSize(painter->font(), kRepoNameFontSize));
+    QRect repo_name_rect(repo_name_pos, QSize(repo_name_width, repo_name_height));
+    painter->save();
     painter->setPen(QColor(selected ? kRepoNameColorHighlighted : kRepoNameColor));
     painter->setFont(changeFontSize(painter->font(), kRepoNameFontSize));
     painter->drawText(repo_name_rect,
@@ -234,12 +259,6 @@ void RepoItemDelegate::paintRepoItem(QPainter *painter,
     painter->restore();
 
     // Paint repo description
-    painter->save();
-    QPoint repo_desc_pos = repo_name_rect.bottomLeft() + QPoint(0, 5);
-    QRect repo_desc_rect(repo_desc_pos, QSize(repo_name_width, kRepoNameHeight));
-    painter->setFont(changeFontSize(painter->font(), kTimestampFontSize));
-    painter->setPen(QColor(selected ? kTimestampColorHighlighted : kTimestampColor));
-
     QString description;
     const LocalRepo& r = item->localRepo();
     if (r.isValid()) {
@@ -268,7 +287,12 @@ void RepoItemDelegate::paintRepoItem(QPainter *painter,
     if (description.isEmpty()) {
         description = translateCommitTime(repo.mtime);
     }
-
+    painter->save();
+    QPoint repo_desc_pos = repo_name_rect.bottomLeft() + QPoint(0, 5);
+    int repo_desc_height = ::textHeightInFont(description, changeFontSize(painter->font(), kRepoDescriptionFontSizeAlpha));
+    QRect repo_desc_rect(repo_desc_pos, QSize(repo_name_width, repo_desc_height));
+    painter->setFont(changeFontSize(painter->font(), kRepoDescriptionFontSizeAlpha));
+    painter->setPen(QColor(selected ? kTimestampColorHighlighted : kTimestampColor));
     painter->drawText(repo_desc_rect,
                       Qt::AlignLeft | Qt::AlignTop | Qt::TextWordWrap,
                       fitTextToWidth(description, painter->font(), repo_name_width),
@@ -287,14 +311,19 @@ void RepoItemDelegate::paintRepoItem(QPainter *painter,
         extra_description += tr(", %1").arg(repo.owner.split('@').front());
 
     if (!extra_description.isEmpty()) {
-        painter->save();
-
         int width = option.rect.topRight().x() - 40 - repo_desc_rect.topRight().x();
         if (width < 3)
-          width = 3;
-        QPoint repo_owner_pos = repo_desc_rect.topRight();
-        QRect repo_owner_rect(repo_owner_pos, QSize(width, kRepoNameHeight));
-        painter->setFont(changeFontSize(painter->font(), kOwnerFontSize));
+            width = 3;
+        int vertical_margin_between_desc_and_extra_desc = 0;
+#ifdef Q_OS_LINUX
+        vertical_margin_between_desc_and_extra_desc = 3;
+#endif
+        QPoint repo_owner_pos = repo_desc_rect.topRight() + QPoint(0, vertical_margin_between_desc_and_extra_desc);
+        int extra_desc_height = ::textHeightInFont(extra_description, changeFontSize(painter->font(), kOwnerFontSizeAlpha));
+        QRect repo_owner_rect(repo_owner_pos, QSize(width, extra_desc_height));
+
+        painter->save();
+        painter->setFont(changeFontSize(painter->font(), kOwnerFontSizeAlpha));
         painter->setPen(QColor(selected ? kTimestampColorHighlighted : kTimestampColor));
         painter->drawText(repo_owner_rect,
                           Qt::AlignLeft | Qt::AlignTop | Qt::TextWordWrap,
@@ -305,14 +334,11 @@ void RepoItemDelegate::paintRepoItem(QPainter *painter,
     }
 
     // Paint repo status icon
-    QPoint status_icon_pos = option.rect.topRight() - QPoint(40, 0);
-    status_icon_pos.setY(option.rect.center().y() - (kRepoStatusIconHeight / 2));
-    QRect status_icon_rect(status_icon_pos, QSize(kRepoStatusIconWidth, kRepoStatusIconHeight));
+    QPoint status_icon_pos = option.rect.topRight() - QPoint(41, 0);
+    status_icon_pos.setY(option.rect.center().y() - (kRepoStatusIconHeightAlpha / 2));
+    QRect status_icon_rect(status_icon_pos, QSize(kRepoStatusIconWidthAlpha, kRepoStatusIconHeightAlpha));
 
-    QPixmap status_icon_pixmap = getSyncStatusIcon(item).pixmap(scale_factor * status_icon_rect.size());
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
-    status_icon_pixmap.setDevicePixelRatio(scale_factor);
-#endif // QT5
+    QPixmap status_icon_pixmap = getSyncStatusIcon(item).pixmap(status_icon_rect.size());
 
     painter->save();
     painter->drawPixmap(status_icon_rect, status_icon_pixmap);
@@ -356,18 +382,21 @@ void RepoItemDelegate::paintRepoCategoryItem(QPainter *painter,
         indent_left = kRepoCategoryIndicatorWidth + kMarginBetweenIndicatorAndName;
     }
 
-    QRect indicator_rect(option.rect.topLeft() + QPoint(kMarginLeft + indent_left, 0),
+    int repo_category_text_height = ::textHeightInFont(item->name(), changeFontSize(option.font, kRepoCategoryNameFontSize));
+    int vertical_padding_between_indicator_and_name = (repo_category_text_height - kRepoCategoryIndicatorHeight) / 2;
+
+    QRect indicator_rect(option.rect.topLeft() +
+                         QPoint(kMarginLeft + indent_left,
+                                kRepoCategoryVerticalMargin + vertical_padding_between_indicator_and_name),
                          QSize(kRepoCategoryIndicatorWidth, kRepoCategoryIndicatorHeight));
     // get the device pixel radio from current painter device
     int scale_factor = 1;
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
     scale_factor = painter->device()->devicePixelRatio();
 #endif // QT5
-    QIcon icon(expanded? awesome->icon(icon_caret_down, kRepoCategoryIndicatorColor) : awesome->icon(icon_caret_right, kRepoCategoryIndicatorColor));
-    QPixmap icon_pixmap(icon.pixmap(QSize(kRepoCategoryIndicatorWidth, kRepoCategoryIndicatorWidth) * scale_factor));
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
-    icon_pixmap.setDevicePixelRatio(scale_factor);
-#endif // QT5
+    QIcon icon(expanded ? awesome->icon(icon_caret_down, kRepoCategoryIndicatorColor)
+                        : awesome->icon(icon_caret_right, kRepoCategoryIndicatorColor));
+    QPixmap icon_pixmap(icon.pixmap(QSize(kRepoCategoryIndicatorWidth, kRepoCategoryIndicatorWidth)));
     painter->drawPixmap(indicator_rect, icon_pixmap);
     painter->restore();
 
@@ -391,12 +420,14 @@ void RepoItemDelegate::paintRepoCategoryItem(QPainter *painter,
         }
     }
     const QString category_count_text = QString::number(synced_repos) + "/" + QString::number(item->matchedReposCount());
-    const int category_count_width = ::textWidthInFont(category_count_text, changeFontSize(option.font, kRepoCategoryCountFontSize));
+    const int category_count_width = ::textWidthInFont(category_count_text, changeFontSize(option.font, kRepoCategoryCountFontSizeAlpha));
 
     painter->save();
-    QPoint category_name_pos = indicator_rect.topRight() + QPoint(kMarginBetweenIndicatorAndName, 0);
+    QPoint category_name_pos = indicator_rect.topRight() +
+                               QPoint(kMarginBetweenIndicatorAndName, - vertical_padding_between_indicator_and_name);
     QRect category_name_rect(category_name_pos,
-                             option.rect.bottomRight() - QPoint(kPadding + category_count_width + kRepoCategoryCountMarginRight, 0));
+                             option.rect.bottomRight() - QPoint(kPadding + category_count_width + kRepoCategoryCountMarginRightAlpha,
+                                                                kRepoCategoryVerticalMargin));
     painter->setPen(QColor(kRepoCategoryColor));
     painter->setFont(changeFontSize(option.font, kRepoCategoryNameFontSize));
     painter->drawText(category_name_rect,
@@ -406,9 +437,12 @@ void RepoItemDelegate::paintRepoCategoryItem(QPainter *painter,
 
     // Paint category count
     painter->save();
-    QPoint category_count_pos = option.rect.topRight() + QPoint(-category_count_width - kPadding - kRepoCategoryCountMarginRight, 0);
-    QRect category_count_rect(category_count_pos, option.rect.bottomRight() - QPoint(kPadding + kRepoCategoryCountMarginRight, 0));
-    painter->setFont(changeFontSize(option.font, kRepoCategoryCountFontSize));
+    QPoint category_count_pos = option.rect.topRight() +
+                                QPoint(-category_count_width - kRepoCategoryCountMarginRightAlpha, 0);
+    category_count_pos.setY(category_name_pos.y());
+    QRect category_count_rect(category_count_pos,
+                              option.rect.bottomRight() - QPoint(kRepoCategoryCountMarginRightAlpha, kRepoCategoryVerticalMargin));
+    painter->setFont(changeFontSize(option.font, kRepoCategoryCountFontSizeAlpha));
     painter->setPen(QColor(kRepoCategoryCountColor));
     painter->drawText(category_count_rect,
                       Qt::AlignLeft | Qt::AlignTop,
@@ -422,16 +456,16 @@ QIcon RepoItemDelegate::getSyncStatusIcon(const RepoItem *item) const
     const LocalRepo& repo = item->localRepo();
     QString icon;
     if (!repo.isValid()) {
-        icon = "cloud";
+        icon = "cloud-unsynced";
     } else if (!seafApplet->settingsManager()->autoSync()) {
         icon = "pause";
     } else {
         switch (repo.sync_state) {
         case LocalRepo::SYNC_STATE_DONE:
-            icon = "done";
+            icon = "cloud-synced";
             break;
         case LocalRepo::SYNC_STATE_ING:
-            icon = "rotate";
+            icon = "cloud-sync";
             break;
         case LocalRepo::SYNC_STATE_ERROR:
             icon = "exclamation";
