@@ -44,7 +44,6 @@ namespace {
 
 const char *kRepoTreeViewSettingsGroup = "RepoTreeView";
 const char *kRepoTreeViewSettingsExpandedCategories = "expandedCategories";
-const int kRepoCategoryIndicatorWidth = 16;
 const char *kSyncIntervalProperty = "sync-interval";
 
 // A Helper Class to copy file
@@ -193,9 +192,6 @@ QMenu* RepoTreeView::prepareContextMenu(const RepoItem *item)
         menu->addAction(toggle_auto_sync_action_);
         menu->addAction(sync_now_action_);
         menu->addAction(set_sync_interval_action_);
-#if defined(Q_OS_WIN32)
-        menu->addAction(map_netdrive_action_);
-#endif
         menu->addSeparator();
     }
 
@@ -237,7 +233,6 @@ void RepoTreeView::updateRepoActions()
         unsync_action_->setEnabled(false);
         resync_action_->setEnabled(false);
         set_sync_interval_action_->setEnabled(false);
-        map_netdrive_action_->setEnabled(false);
         toggle_auto_sync_action_->setEnabled(false);
         view_on_web_action_->setEnabled(false);
         open_in_filebrowser_action_->setEnabled(false);
@@ -271,9 +266,6 @@ void RepoTreeView::updateRepoActions()
         set_sync_interval_action_->setData(QVariant::fromValue(local_repo));
         set_sync_interval_action_->setEnabled(true);
 
-        map_netdrive_action_->setData(QVariant::fromValue(local_repo));
-        map_netdrive_action_->setEnabled(true);
-
         if (seafApplet->settingsManager()->autoSync()) {
             toggle_auto_sync_action_->setData(QVariant::fromValue(local_repo));
             toggle_auto_sync_action_->setEnabled(true);
@@ -307,7 +299,6 @@ void RepoTreeView::updateRepoActions()
         unsync_action_->setEnabled(false);
         resync_action_->setEnabled(false);
         set_sync_interval_action_->setEnabled(false);
-        map_netdrive_action_->setEnabled(false);
         toggle_auto_sync_action_->setEnabled(false);
     }
 
@@ -332,7 +323,8 @@ QStandardItem* RepoTreeView::getRepoItem(const QModelIndex &index) const
     }
     QSortFilterProxyModel *proxy = (QSortFilterProxyModel *)model();
     RepoTreeModel *tree_model = (RepoTreeModel *)(proxy->sourceModel());
-    QStandardItem *item = tree_model->itemFromIndex(proxy->mapToSource(index));
+    const QModelIndex mapped_index = proxy->mapToSource(index);
+    QStandardItem *item = tree_model->itemFromIndex(mapped_index);
 
     if (item->type() != REPO_ITEM_TYPE &&
         item->type() != REPO_CATEGORY_TYPE) {
@@ -440,14 +432,6 @@ void RepoTreeView::createActions()
     set_sync_interval_action_->setStatusTip(tr("unsync and resync this library"));
 
     connect(set_sync_interval_action_, SIGNAL(triggered()), this, SLOT(setRepoSyncInterval()));
-
-    map_netdrive_action_ = new QAction(tr("&Map as a network drive"), this);
-    map_netdrive_action_->setIcon(QIcon(":/images/disk.png"));
-    map_netdrive_action_->setStatusTip(tr("map as a network drive"));
-
-#if defined(Q_OS_WIN32)
-    connect(map_netdrive_action_, SIGNAL(triggered()), this, SLOT(mapLibraryAsNetworkDrive()));
-#endif
 }
 
 void RepoTreeView::downloadRepo()
@@ -584,7 +568,11 @@ void RepoTreeView::openInFileBrowser()
 
 bool RepoTreeView::viewportEvent(QEvent *event)
 {
-    if (event->type() != QEvent::ToolTip && event->type() != QEvent::WhatsThis && event->type() != QEvent::MouseButtonPress && event->type() != QEvent::MouseButtonRelease) {
+    if (event->type() != QEvent::ToolTip &&
+        event->type() != QEvent::WhatsThis &&
+        event->type() != QEvent::MouseButtonPress &&
+        event->type() != QEvent::MouseButtonRelease)
+    {
         return QTreeView::viewportEvent(event);
     }
 
@@ -602,14 +590,24 @@ bool RepoTreeView::viewportEvent(QEvent *event)
 
     // handle the event in the top
     const QModelIndex top_index = indexAt(QPoint(0, 0));
-    if (event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonRelease) {
-        if (index == top_index && top_index.parent().isValid() && viewport_pos.y() <= kRepoCategoryIndicatorWidth) {
+    if (event->type() == QEvent::MouseButtonPress ||
+        event->type() == QEvent::MouseButtonRelease)
+    {
+        QSortFilterProxyModel *proxy = (QSortFilterProxyModel *)model();
+        RepoTreeModel *tree_model = (RepoTreeModel *)(proxy->sourceModel());
+
+        if (index == top_index &&
+            top_index.parent().isValid() &&
+            viewport_pos.y() <= tree_model->repo_category_height)
+        {
             QMouseEvent *ev = static_cast<QMouseEvent*>(event);
-            if (!(ev->buttons() & Qt::LeftButton))
+            if (!(ev->buttons() & Qt::LeftButton)) {
                 return true;
-            const QModelIndex parent = top_index.parent();
-            setExpanded(parent, !isExpanded(parent));
-            return true;
+            } else {
+                const QModelIndex parent = top_index.parent();
+                setExpanded(parent, !isExpanded(parent));
+                return true;
+            }
         }
         return QTreeView::viewportEvent(event);
     }
@@ -1051,109 +1049,3 @@ void RepoTreeView::setRepoSyncInterval()
     seafApplet->rpcClient()->setRepoProperty(
         local_repo.id, kSyncIntervalProperty, QString::number(interval));
 }
-
-#if defined(Q_OS_WIN32)
-static bool findNextAvailableDrive(QString* drive)
-{
-    DWORD bitmap = GetLogicalDrives();
-    if (bitmap == 0) {
-        return false;
-    }
-    char disk = 'A';
-    DWORD mask = 1;
-    while (disk <= 'Z') {
-        if (!(bitmap & mask)) {
-            *drive = QString(disk) + ":";
-            return true;
-        }
-        mask = mask << 1;
-        disk += 1;
-    }
-    return false;
-}
-
-static QString getDriveNetworkPath(const QString& drive)
-{
-    wchar_t wbuf[8192];
-    DWORD size = sizeof(wbuf) / sizeof(wchar_t);
-    if (WNetGetConnectionW(drive.toStdWString().c_str(), wbuf, &size) ==
-        NO_ERROR) {
-        return QString::fromWCharArray(wbuf);
-    }
-    return QString();
-}
-
-static bool alreadyMappedAsNetworkDrive(const QString& path, QString* drive)
-{
-    DWORD bitmap = GetLogicalDrives();
-    if (bitmap == 0) {
-        return false;
-    }
-    char disk = 'A';
-    DWORD mask = 1;
-    while (disk <= 'Z') {
-        if (bitmap & mask) {
-            QString drive_root = QString(disk) + ":";
-            if (getDriveNetworkPath(drive_root) == path) {
-                *drive = drive_root;
-                return true;
-            }
-        }
-        mask = mask << 1;
-        disk += 1;
-    }
-    return false;
-}
-
-static QString formatErrorMessage(DWORD error_code)
-{
-    if (error_code == 0) {
-        return QObject::tr("unknown error");
-    }
-    wchar_t wbuf[4096] = {0};
-    ::FormatMessageW(FORMAT_MESSAGE_FROM_SYSTEM,
-                    NULL,
-                    error_code,
-                    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-                    wbuf,
-                    (sizeof(wbuf) / sizeof(wchar_t)) - 1,
-                    NULL);
-    return QString::fromWCharArray(wbuf);
-}
-
-void RepoTreeView::mapLibraryAsNetworkDrive()
-{
-    LocalRepo local_repo =
-        qvariant_cast<LocalRepo>(set_sync_interval_action_->data());
-    QString worktree = QDir::toNativeSeparators(local_repo.worktree);
-    // Path format is \\localhost\c$\folder
-    QString net_path = QString("\\\\localhost\\%1$\\%2")
-                           .arg(worktree.left(1).toLower())
-                           .arg(worktree.mid(3));
-
-    QString mapped_drive;
-    if (alreadyMappedAsNetworkDrive(net_path, &mapped_drive)) {
-        QDesktopServices::openUrl(QUrl::fromLocalFile(mapped_drive));
-        return;
-    }
-
-    if (!findNextAvailableDrive(&mapped_drive)) {
-        seafApplet->warningBox(tr("No available windows drive letter"));
-        return;
-    }
-
-    qWarning("path is %s", net_path.toUtf8().data());
-
-    int code = QProcess::execute("net", QStringList() << "use" << mapped_drive
-                                                      << net_path);
-    if (code != 0) {
-        seafApplet->warningBox(tr("Operation failed: %1").arg(formatErrorMessage(code)));
-    } else {
-        QDesktopServices::openUrl(QUrl::fromLocalFile(mapped_drive));
-    }
-}
-#else
-void RepoTreeView::mapLibraryAsNetworkDrive()
-{
-}
-#endif // Q_OS_WIN32
