@@ -65,7 +65,6 @@ private slots:
     void saveExpandedCategries();
     void resyncRepo();
     void setRepoSyncInterval();
-    void mapLibraryAsNetworkDrive();
 
     void checkRootPermDone();
     void uploadFileStart(FileUploadTask *task);
@@ -113,7 +112,6 @@ private:
     QAction *cancel_download_action_;
     QAction *resync_action_;
     QAction *set_sync_interval_action_;
-    QAction *map_netdrive_action_;
 
     QSet<QString> expanded_categroies_;
 
